@@ -20,6 +20,7 @@ from absl.testing import parameterized
 
 from acme.adders.reverb import episode as adders
 from acme.adders.reverb import test_utils
+import numpy as np
 
 
 class EpisodeAdderTest(test_utils.AdderTestMixin, parameterized.TestCase):
@@ -41,8 +42,7 @@ class EpisodeAdderTest(test_utils.AdderTestMixin, parameterized.TestCase):
 
   @parameterized.parameters(2, 10, 50)
   def test_max_sequence_length(self, max_sequence_length):
-    client = test_utils.FakeClient()
-    adder = adders.EpisodeAdder(client, max_sequence_length)
+    adder = adders.EpisodeAdder(self.client, max_sequence_length)
 
     first, steps = test_utils.make_trajectory(range(max_sequence_length + 1))
     adder.add_first(first)
@@ -52,7 +52,7 @@ class EpisodeAdderTest(test_utils.AdderTestMixin, parameterized.TestCase):
     # We should have max_sequence_length-1 timesteps that have been written,
     # where the -1 is due to the dangling observation (ie we have actually
     # seen max_sequence_length observations).
-    self.assertLen(client.writers[0].timesteps, max_sequence_length - 1)
+    self.assertEqual(self.client.writer.episode_steps, max_sequence_length - 1)
 
     # Adding one more step should raise an error.
     with self.assertRaises(ValueError):
@@ -60,25 +60,49 @@ class EpisodeAdderTest(test_utils.AdderTestMixin, parameterized.TestCase):
       adder.add(action, step)
 
     # Since the last insert failed it should not affect the internal state.
-    self.assertLen(client.writers[0].timesteps, max_sequence_length - 1)
+    self.assertEqual(self.client.writer.episode_steps, max_sequence_length - 1)
 
-  def test_delta_encoded_and_chunk_length(self):
-    max_sequence_length = 5
-    client = test_utils.FakeClient()
+  @parameterized.parameters((2, 1), (10, 2), (50, 5))
+  def test_padding(self, max_sequence_length, padding):
     adder = adders.EpisodeAdder(
-        client,
-        max_sequence_length,
-        delta_encoded=True,
-        chunk_length=max_sequence_length)
+        self.client,
+        max_sequence_length + padding,
+        padding_fn=np.zeros)
 
-    # Add an episode.
-    first, steps = test_utils.make_trajectory(range(max_sequence_length))
-    adder.add_first(first)
-    for action, step in steps:
-      adder.add(action, step)
+    # Create a simple trajectory to add.
+    observations = range(max_sequence_length)
+    first, steps = test_utils.make_trajectory(observations)
 
-    self.assertTrue(client.writers[0].delta_encoded)
-    self.assertEqual(max_sequence_length, client.writers[0].chunk_length)
+    expected_episode = test_utils.make_sequence(observations)
+    for _ in range(padding):
+      expected_episode.append((0, 0, 0.0, 0.0, False, ()))
+
+    self.run_test_adder(
+        adder=adder,
+        first=first,
+        steps=steps,
+        expected_items=[expected_episode])
+
+  @parameterized.parameters((2, 1), (10, 2), (50, 5))
+  def test_nonzero_padding(self, max_sequence_length, padding):
+    adder = adders.EpisodeAdder(
+        self.client,
+        max_sequence_length + padding,
+        padding_fn=lambda s, d: np.zeros(s, d) - 1)
+
+    # Create a simple trajectory to add.
+    observations = range(max_sequence_length)
+    first, steps = test_utils.make_trajectory(observations)
+
+    expected_episode = test_utils.make_sequence(observations)
+    for _ in range(padding):
+      expected_episode.append((-1, -1, -1.0, -1.0, False, ()))
+
+    self.run_test_adder(
+        adder=adder,
+        first=first,
+        steps=steps,
+        expected_items=[expected_episode])
 
 
 if __name__ == '__main__':
