@@ -16,4 +16,7 @@
 """Implementation of a deep Q-networks (DQN) agent."""
 
 from acme.agents.jax.dqn.agent import DQN
+from acme.agents.jax.dqn.builder import default_behavior_policy
+from acme.agents.jax.dqn.builder import DQNBuilder
+from acme.agents.jax.dqn.config import DQNConfig
 from acme.agents.jax.dqn.learning import DQNLearner
