@@ -18,6 +18,7 @@
 from typing import Dict, List
 
 import acme
+from acme import types
 from acme.tf import savers as tf2_savers
 from acme.tf import utils as tf2_utils
 from acme.utils import counting
@@ -31,8 +32,7 @@ class BCLearner(acme.Learner, tf2_savers.TFSaveable):
   """BC learner.
 
   This is the learning component of a BC agent. IE it takes a dataset as input
-  and implements update functionality to learn from this dataset. Optionally
-  it takes a replay client as well to allow for updating of priorities.
+  and implements update functionality to learn from this dataset.
   """
 
   def __init__(self,
@@ -45,8 +45,8 @@ class BCLearner(acme.Learner, tf2_savers.TFSaveable):
     """Initializes the learner.
 
     Args:
-      network: the online Q network (the one being optimized)
-      learning_rate: learning rate for the q-network update.
+      network: the BC network (the one being optimized)
+      learning_rate: learning rate for the cross-entropy update.
       dataset: dataset to learn from.
       counter: Counter object for (potentially distributed) counting.
       logger: Logger object for writing logs to.
@@ -79,14 +79,13 @@ class BCLearner(acme.Learner, tf2_savers.TFSaveable):
 
     # Pull out the data needed for updates/priorities.
     inputs = next(self._iterator)
-    o_tm1, a_tm1, r_t, d_t, o_t = inputs.data
-    del r_t, d_t, o_t
+    transitions: types.Transition = inputs.data
 
     with tf.GradientTape() as tape:
       # Evaluate our networks.
-      logits = self._network(o_tm1)
+      logits = self._network(transitions.observation)
       cce = tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True)
-      loss = cce(a_tm1, logits)
+      loss = cce(transitions.action, logits)
 
     gradients = tape.gradient(loss, self._network.trainable_variables)
     self._optimizer.apply(gradients, self._network.trainable_variables)
