@@ -55,12 +55,14 @@ class DQN(agent.Agent):
       importance_sampling_exponent: float = 0.2,
       priority_exponent: float = 0.6,
       n_step: int = 5,
-      epsilon: Optional[tf.Tensor] = None,
+      epsilon: Optional[tf.Variable] = None,
       learning_rate: float = 1e-3,
       discount: float = 0.99,
       logger: loggers.Logger = None,
       checkpoint: bool = True,
       checkpoint_subpath: str = '~/acme/',
+      policy_network: Optional[snt.Module] = None,
+      max_gradient_norm: Optional[float] = None,
   ):
     """Initialize the agent.
 
@@ -88,6 +90,10 @@ class DQN(agent.Agent):
       logger: logger object to be used by learner.
       checkpoint: boolean indicating whether to checkpoint the learner.
       checkpoint_subpath: directory for the checkpoint.
+      policy_network: if given, this will be used as the policy network.
+        Otherwise, an epsilon greedy policy using the online Q network will be
+        created. Policy network is used in the actor to sample actions.
+      max_gradient_norm: used for gradient clipping.
     """
 
     # Create a replay server to add data to. This uses no limiter behavior in
@@ -115,13 +121,15 @@ class DQN(agent.Agent):
         batch_size=batch_size,
         prefetch_size=prefetch_size)
 
-    # Use constant 0.05 epsilon greedy policy by default.
-    if epsilon is None:
-      epsilon = tf.Variable(0.05, trainable=False)
-    policy_network = snt.Sequential([
-        network,
-        lambda q: trfl.epsilon_greedy(q, epsilon=epsilon).sample(),
-    ])
+    # Create epsilon greedy policy network by default.
+    if policy_network is None:
+      # Use constant 0.05 epsilon greedy policy by default.
+      if epsilon is None:
+        epsilon = tf.Variable(0.05, trainable=False)
+      policy_network = snt.Sequential([
+          network,
+          lambda q: trfl.epsilon_greedy(q, epsilon=epsilon).sample(),
+      ])
 
     # Create a target network.
     target_network = copy.deepcopy(network)
@@ -143,6 +151,7 @@ class DQN(agent.Agent):
         target_update_period=target_update_period,
         dataset=dataset,
         replay_client=replay_client,
+        max_gradient_norm=max_gradient_norm,
         logger=logger,
         checkpoint=checkpoint)
 
