@@ -19,15 +19,18 @@ This file specifies and documents the notions of `Actor` and `Learner`.
 """
 
 import abc
-from typing import Generic, List, NoReturn, TypeVar
+import itertools
+from typing import Generic, List, Optional, Sequence, TypeVar
 
 from acme import types
 # Internal imports.
+from acme.utils import metrics
 import dm_env
 
 T = TypeVar('T')
 
 
+@metrics.record_class_usage
 class Actor(abc.ABC):
   """Interface for an agent that can act.
 
@@ -97,7 +100,7 @@ class VariableSource(abc.ABC):
   """
 
   @abc.abstractmethod
-  def get_variables(self, names: List[str]) -> List[types.NestedArray]:
+  def get_variables(self, names: Sequence[str]) -> List[types.NestedArray]:
     """Return the named variables as a collection of (nested) numpy arrays.
 
     Args:
@@ -110,6 +113,7 @@ class VariableSource(abc.ABC):
     """
 
 
+@metrics.record_class_usage
 class Worker(abc.ABC):
   """An interface for (potentially) distributed workers."""
 
@@ -118,11 +122,23 @@ class Worker(abc.ABC):
     """Runs the worker."""
 
 
-class Learner(VariableSource, Worker):
+class Saveable(abc.ABC, Generic[T]):
+  """An interface for saveable objects."""
+
+  @abc.abstractmethod
+  def save(self) -> T:
+    """Returns the state from the object to be saved."""
+
+  @abc.abstractmethod
+  def restore(self, state: T):
+    """Given the state, restores the object."""
+
+
+class Learner(VariableSource, Worker, Saveable):
   """Abstract learner object.
 
   This corresponds to an object which implements a learning loop. A single step
-  of learning should be implemented via the `update` method and this step
+  of learning should be implemented via the `step` method and this step
   is generally interacted with via the `run` method which runs update
   continuously.
 
@@ -138,19 +154,16 @@ class Learner(VariableSource, Worker):
   def step(self):
     """Perform an update step of the learner's parameters."""
 
-  def run(self) -> NoReturn:
+  def run(self, num_steps: Optional[int] = None) -> None:
     """Run the update loop; typically an infinite loop which calls step."""
-    while True:
+
+    iterator = range(num_steps) if num_steps is not None else itertools.count()
+
+    for _ in iterator:
       self.step()
 
+  def save(self):
+    raise NotImplementedError('Method "save" is not implemented.')
 
-class Saveable(abc.ABC, Generic[T]):
-  """An interface for saveable objects."""
-
-  @abc.abstractmethod
-  def save(self) -> T:
-    """Returns the state from the object to be saved."""
-
-  @abc.abstractmethod
-  def restore(self, state: T):
-    """Given the state, restores the object."""
+  def restore(self, state):
+    raise NotImplementedError('Method "restore" is not implemented.')
