@@ -19,14 +19,29 @@ from acme.jax.networks.atari import AtariTorso
 from acme.jax.networks.atari import DeepIMPALAAtariNetwork
 from acme.jax.networks.atari import dqn_atari_network
 from acme.jax.networks.atari import R2D2AtariNetwork
+from acme.jax.networks.base import Action
+from acme.jax.networks.base import FeedForwardNetwork
 from acme.jax.networks.base import Logits
+from acme.jax.networks.base import LogProbFn
+from acme.jax.networks.base import LSTMOutputs
+from acme.jax.networks.base import NetworkOutput
+from acme.jax.networks.base import Observation
+from acme.jax.networks.base import Params
 from acme.jax.networks.base import PolicyValueRNN
+from acme.jax.networks.base import PRNGKey
 from acme.jax.networks.base import QNetwork
 from acme.jax.networks.base import RecurrentQNetwork
+from acme.jax.networks.base import SampleFn
 from acme.jax.networks.base import Value
 from acme.jax.networks.continuous import LayerNormMLP
 from acme.jax.networks.continuous import NearZeroInitializedLinear
 from acme.jax.networks.distributional import CategoricalHead
+from acme.jax.networks.distributional import CategoricalValueHead
+from acme.jax.networks.distributional import GaussianMixture
+from acme.jax.networks.distributional import MultivariateNormalDiagHead
+from acme.jax.networks.distributional import NormalTanhDistribution
+from acme.jax.networks.duelling import DuellingMLP
+from acme.jax.networks.multiplexers import CriticMultiplexer
 from acme.jax.networks.policy_value import PolicyValueHead
 from acme.jax.networks.rescaling import ClipToSpec
 from acme.jax.networks.rescaling import TanhToSpec
