@@ -19,20 +19,43 @@ import abc
 from typing import Callable, Tuple
 
 from acme import types
+import dataclasses
 import haiku as hk
 import jax.numpy as jnp
 
 # Commonly-used types.
+PRNGKey = jnp.ndarray
+Observation = types.NestedArray
+Action = types.NestedArray
+Params = types.NestedArray
+NetworkOutput = types.NestedArray
+Action = types.NestedArray
 QValues = jnp.ndarray
 Logits = jnp.ndarray
 Value = jnp.ndarray
 
 # Commonly-used function/network signatures.
-QNetwork = Callable[[types.NestedArray], QValues]
+QNetwork = Callable[[Observation], QValues]
 LSTMOutputs = Tuple[Tuple[Logits, Value], hk.LSTMState]
-PolicyValueRNN = Callable[[types.NestedArray, hk.LSTMState], LSTMOutputs]
-RecurrentQNetwork = Callable[[types.NestedArray, hk.LSTMState],
+PolicyValueRNN = Callable[[Observation, hk.LSTMState], LSTMOutputs]
+RecurrentQNetwork = Callable[[Observation, hk.LSTMState],
                              Tuple[QValues, hk.LSTMState]]
+SampleFn = Callable[[NetworkOutput, PRNGKey], Action]
+LogProbFn = Callable[[NetworkOutput, Action], Logits]
+
+
+@dataclasses.dataclass
+class FeedForwardNetwork:
+  """Holds a pair of pure functions defining a feed-forward network.
+
+  Attributes:
+    init: A pure function: ``params = init(rng, *a, **k)``
+    apply: A pure function: ``out = apply(params, rng, *a, **k)``
+  """
+  # Initializes and returns the networks parameters.
+  init: Callable[..., Params]
+  # Computes and returns the outputs of a forward pass.
+  apply: Callable[..., NetworkOutput]
 
 
 class Module(hk.Module, abc.ABC):
