@@ -19,10 +19,10 @@ import sys
 from typing import Optional
 
 from absl import flags
-from absl.testing import absltest
+from absl.testing import parameterized
 
 
-class TestCase(absltest.TestCase):
+class TestCase(parameterized.TestCase):
   """A custom TestCase which handles FLAG parsing for pytest compatibility."""
 
   def get_tempdir(self, name: Optional[str] = None) -> str:
