@@ -34,6 +34,7 @@ from acme.tf.networks.distributional import MultivariateNormalDiagHead
 from acme.tf.networks.distributional import UnivariateGaussianMixture
 from acme.tf.networks.distributions import DiscreteValuedDistribution
 from acme.tf.networks.duelling import DuellingMLP
+from acme.tf.networks.masked_epsilon_greedy import NetworkWithMaskedEpsilonGreedy
 from acme.tf.networks.multihead import Multihead
 from acme.tf.networks.multiplexers import CriticMultiplexer
 from acme.tf.networks.noise import ClippedGaussian
@@ -54,4 +55,11 @@ from acme.tf.networks.vision import ResNetTorso
 # For backwards compatibility.
 GaussianMixtureHead = UnivariateGaussianMixture
 
+try:
+  # pylint: disable=g-bad-import-order,g-import-not-at-top
+  from acme.tf.networks.legal_actions import MaskedSequential
+  from acme.tf.networks.legal_actions import EpsilonGreedy
+except ImportError:
+  pass
+
 # Internal imports.
