@@ -55,7 +55,7 @@ class Checkpointer:
   This can be used to checkpoint any object with trackable state (e.g.
   tensorflow variables or modules); see tf.train.Checkpoint for
   details. Objects inheriting from tf.train.experimental.PythonState can also
-  be checkointed.
+  be checkpointed.
 
   Typically people use Checkpointer to make sure that they can correctly recover
   from a machine going down during learning. For more permanent storage of self-
@@ -171,13 +171,13 @@ class CheckpointingRunner(core.Worker):
   """Wrap an object and expose a run method which checkpoints periodically.
 
   This internally creates a Checkpointer around `wrapped` object and exposes
-  all of the methods of `wrapped`. Additionally, anay `**kwargs` passed to the
+  all of the methods of `wrapped`. Additionally, any `**kwargs` passed to the
   runner are forwarded to the internal Checkpointer.
   """
 
   def __init__(
       self,
-      wrapped: Union[Checkpointable, core.Saveable, core.Learner, TFSaveable],
+      wrapped: Union[Checkpointable, core.Saveable, TFSaveable],
       *,
       time_delta_minutes: int = 30,
       **kwargs,
@@ -197,9 +197,6 @@ class CheckpointingRunner(core.Worker):
         time_delta_minutes=time_delta_minutes,
         **kwargs)
 
-  def run(self):
-    """Runs the checkpointer."""
-
     # Handle preemption signal. Note that this must happen in the main thread.
     def _signal_handler(signum: signal.Signals, frame):
       del signum, frame
@@ -214,15 +211,20 @@ class CheckpointingRunner(core.Worker):
           'This probably means we are not running in the main thread. '
           'Proceeding without checkpointing-on-preemption.')
 
+  def step(self):
     if isinstance(self._wrapped, core.Learner):
       # Learners have a step() method, so alternate between that and ckpt call.
-      while True:
-        self._wrapped.step()
-        self._checkpointer.save()
+      self._wrapped.step()
+      self._checkpointer.save()
     else:
       # Wrapped object doesn't have a run method; set our run method to ckpt.
       self.checkpoint()
 
+  def run(self):
+    """Runs the checkpointer."""
+    while True:
+      self.step()
+
   def __dir__(self):
     return dir(self._wrapped)
 
@@ -230,16 +232,15 @@ class CheckpointingRunner(core.Worker):
     return getattr(self._wrapped, name)
 
   def checkpoint(self):
-    while True:
-      self._checkpointer.save()
-      time.sleep(self._time_delta_minutes * 60)
+    self._checkpointer.save()
+    time.sleep(self._time_delta_minutes * 60)
 
 
 class Snapshotter:
   """Convenience class for periodically snapshotting.
 
   Objects which can be snapshotted are limited to Sonnet or tensorflow Modules
-  which implement a a __call__ method. This will save the module's graph and
+  which implement a __call__ method. This will save the module's graph and
   variables such that they can be loaded later using `tf.saved_model.load`. See
   https://www.tensorflow.org/guide/saved_model for more details.
 
