@@ -15,7 +15,7 @@
 
 """Common types used throughout Acme."""
 
-from typing import Any, Callable, Iterable, Mapping, Union
+from typing import Any, Callable, Iterable, Mapping, NamedTuple, Union
 from acme import specs
 
 # Define types for nested arrays and tensors.
@@ -23,14 +23,26 @@ from acme import specs
 NestedArray = Any
 NestedTensor = Any
 
+# pytype: disable=not-supported-yet
 NestedSpec = Union[
     specs.Array,
     Iterable['NestedSpec'],
-    Mapping[Any, 'NestedSpec'],  # pytype: disable=not-supported-yet
+    Mapping[Any, 'NestedSpec'],
 ]
+# pytype: enable=not-supported-yet
 
 # TODO(b/144763593): Replace all instances of nest with the tensor/array types.
 Nest = Union[NestedArray, NestedTensor, NestedSpec]
 
 TensorTransformation = Callable[[NestedTensor], NestedTensor]
 TensorValuedCallable = Callable[..., NestedTensor]
+
+
+class Transition(NamedTuple):
+  """Container for a transition."""
+  observation: NestedArray
+  action: NestedArray
+  reward: NestedArray
+  discount: NestedArray
+  next_observation: NestedArray
+  extras: NestedArray = ()
