@@ -30,16 +30,22 @@ from acme.utils.loggers import base
 
 
 class CSVLogger(base.Logger):
-  """Standard CSV logger."""
+  """Standard CSV logger.
+
+  The fields are inferred from the first call to write() and any additional
+  fields afterwards are ignored.
+  """
 
   _open = open
 
   def __init__(self,
                directory_or_file: Union[str, TextIO] = '~/acme',
                label: str = '',
-               time_delta: float = 0.):
+               time_delta: float = 0.,
+               add_uid: bool = True):
     self._last_log_time = time.time() - time_delta
     self._time_delta = time_delta
+    self._add_uid = add_uid
     self._writer = None
     self._file = self._create_file(directory_or_file, label)
     logging.info('Logging to %s', self.file_path)
@@ -49,7 +55,7 @@ class CSVLogger(base.Logger):
     """Opens a file if input is a directory or use existing file."""
     if isinstance(directory_or_file, str):
       directory = paths.process_path(
-          directory_or_file, 'logs', label, add_uid=True)
+          directory_or_file, 'logs', label, add_uid=self._add_uid)
       file_path = os.path.join(directory, 'logs.csv')
       file = self._open(file_path, mode='a')
     else:
@@ -64,7 +70,6 @@ class CSVLogger(base.Logger):
 
   def write(self, data: base.LoggingData):
     """Writes a `data` into a row of comma-separated values."""
-
     # Only log if `time_delta` seconds have passed since last logging event.
     now = time.time()
     if now - self._last_log_time < self._time_delta:
@@ -73,9 +78,12 @@ class CSVLogger(base.Logger):
 
     # Append row to CSV.
     data = base.to_numpy(data)
+    # Use fields from initial `data` to create the header. If extra fields are
+    # present in subsequent `data`, we ignore them.
     if not self._writer:
-      keys = sorted(data.keys())
-      self._writer = csv.DictWriter(self._file, fieldnames=keys)
+      fields = sorted(data.keys())
+      self._writer = csv.DictWriter(self._file, fieldnames=fields,
+                                    extrasaction='ignore')
       self._writer.writeheader()
     self._writer.writerow(data)
 
