@@ -24,5 +24,12 @@ from acme.wrappers.gym_wrapper import GymAtariAdapter
 from acme.wrappers.gym_wrapper import GymWrapper
 from acme.wrappers.observation_action_reward import ObservationActionRewardWrapper
 from acme.wrappers.single_precision import SinglePrecisionWrapper
+from acme.wrappers.step_limit import StepLimitWrapper
 from acme.wrappers.video import MujocoVideoWrapper
 from acme.wrappers.video import VideoWrapper
+
+try:
+  # pylint: disable=g-import-not-at-top
+  from acme.wrappers.open_spiel_wrapper import OpenSpielWrapper
+except ImportError:
+  pass
