@@ -24,15 +24,35 @@ import numpy as np
 
 SKIP_GYM_TESTS = False
 SKIP_GYM_MESSAGE = 'gym not installed.'
+SKIP_ATARI_TESTS = False
+SKIP_ATARI_MESSAGE = ''
 
 try:
   # pylint: disable=g-import-not-at-top
   from acme.wrappers import gym_wrapper
   import gym
+  # pylint: enable=g-import-not-at-top
 except ModuleNotFoundError:
   SKIP_GYM_TESTS = True
 
 
+try:
+  import atari_py  # pylint: disable=g-import-not-at-top
+  atari_py.get_game_path('pong')
+except ModuleNotFoundError as e:
+  SKIP_ATARI_TESTS = True
+  SKIP_ATARI_MESSAGE = str(e)
+except Exception as e:  # pylint: disable=broad-except
+  # This exception is raised by atari_py.get_game_path('pong') if the Atari ROM
+  # file has not been installed.
+  SKIP_ATARI_TESTS = True
+  SKIP_ATARI_MESSAGE = str(e)
+  del atari_py
+else:
+  del atari_py
+
+
+@unittest.skipIf(SKIP_ATARI_TESTS, SKIP_ATARI_MESSAGE)
 @unittest.skipIf(SKIP_GYM_TESTS, SKIP_GYM_MESSAGE)
 class AtariWrapperTest(parameterized.TestCase):
 
